'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = enableGracefulClose;

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var log = (0, _debug2.default)('graceful');
var wait = function wait(delay) {
  return new Promise(function (r) {
    return setTimeout(r, delay);
  });
};

function enableGracefulClose(server, userOptions, callback) {
  if (typeof userOptions === 'function') {
    return enableGracefulClose(server, {}, userOptions);
  }

  var options = _extends({
    timeout: 10 * 1000 }, userOptions);

  var connections = new Set();
  var closing = false;

  server.on('connection', function (connection) {
    connections.add(connection);
    connection.on('close', function () {
      return connections.delete(connection);
    });
  });

  function close() {
    var _this = this;

    log('Graceful close start');
    if (closing) {
      log('Server closing is already in the progress');
      return;
    }

    closing = true;

    // server can be opened and closed again
    log('Closing listening');
    server.close(_asyncToGenerator(regeneratorRuntime.mark(function _callee() {
      var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, connection;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              log('Server listening is closed');
              closing = false;

              if (!connections.size) {
                _context.next = 25;
                break;
              }

              _context.next = 5;
              return wait(options.timeout);

            case 5:
              _iteratorNormalCompletion = true;
              _didIteratorError = false;
              _iteratorError = undefined;
              _context.prev = 8;


              for (_iterator = connections[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                connection = _step.value;

                connection.end();
              }

              _context.next = 16;
              break;

            case 12:
              _context.prev = 12;
              _context.t0 = _context['catch'](8);
              _didIteratorError = true;
              _iteratorError = _context.t0;

            case 16:
              _context.prev = 16;
              _context.prev = 17;

              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }

            case 19:
              _context.prev = 19;

              if (!_didIteratorError) {
                _context.next = 22;
                break;
              }

              throw _iteratorError;

            case 22:
              return _context.finish(19);

            case 23:
              return _context.finish(16);

            case 24:
              connections.clear();

            case 25:
              if (!callback) {
                _context.next = 28;
                break;
              }

              _context.next = 28;
              return callback();

            case 28:

              process.exit(0);

            case 29:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[8, 12, 16, 24], [17,, 19, 23]]);
    })));
  }

  process.on('SIGINT', close);

  return close;
}